use Test::More;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Installer::API::CVSTagHandler;
use strict;

sub Main {
    my $TagHandler = DE_EPAGES::Installer::API::CVSTagHandler->new();

    Cleanup($TagHandler);

    my $aChangedFiles = $TagHandler->getChangedFiles(
        'release_test_CVSHandler_old',
        'patch_test_CVSHandler_new',
        {'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t'}
    );

    ok (scalar @$aChangedFiles == 0, 'no result if tag doesnt exist.');

    #Test if files changed, set patch tag to 1.3.2.1
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.1', '-F' => '' },
        [
            'patch_test_CVSHandler_new',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );

    #get results
    $aChangedFiles = $TagHandler->getChangedFiles(
        'release_test_CVSHandler_old',
        'patch_test_CVSHandler_new',
        {'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t'}
    );
    my $aChangedFilesUnchangedContent = $TagHandler->getChangedFilesWithUnchangedContent(
        'release_test_CVSHandler_old',
        'patch_test_CVSHandler_new',
        {'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t'}
    );

    #check if changed file is find
    ok (
        (
            (scalar @$aChangedFiles == 1) and
            ($aChangedFiles->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt')
        ),
        'result found in changed files.'
    );

    #check if no file with unchanged content is find
    ok (scalar @$aChangedFilesUnchangedContent == 0, 'no result with unchanged content exist.');

    #Test for unchanged content set patch tag to 1.3.2.2
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.2', '-F' => '' },
        [
            'patch_test_CVSHandler_new',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );

    #get results
    $aChangedFiles = $TagHandler->getChangedFiles(
        'release_test_CVSHandler_old',
        'patch_test_CVSHandler_new',
        {'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t'}
    );
    $aChangedFilesUnchangedContent = $TagHandler->getChangedFilesWithUnchangedContent(
        'release_test_CVSHandler_old',
        'patch_test_CVSHandler_new',
        {'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t'}
    );

    #check if changed file is find
    ok (
        (
            (scalar @$aChangedFiles == 1) and
            ($aChangedFiles->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt')
        ),
        'result found in changed files.'
    );

    #check if changed file with unchanged content is find
    ok (
        (
            (scalar @$aChangedFilesUnchangedContent == 1) and
            ($aChangedFilesUnchangedContent->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt')
        ),
        'result found in changed files with unchanged content.'
    );

    #removing patch tag
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_new');

    #check if result is empty
    $aChangedFiles = $TagHandler->getChangedFiles(
        'release_test_CVSHandler_old',
        'patch_test_CVSHandler_new',
        {'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t'}
    );

    #check if result is empty so this is a indirect proof that the patch tag is gone
    ok (scalar @$aChangedFiles == 0, 'patch tag is removed');

    #check for patch affected file names
    ok ($TagHandler->patchAffectedFile('RND/epages/epages5/AllOS/Patch/src/Shared/Config/Scheduler.conf-patch'), 'patch affected file.');
    ok ($TagHandler->patchAffectedFile('\Cartridges\DE_EPAGES\Calendar\Data\Private\BO\sprite_Calendar.css'), 'patch affected file.');
    ok ($TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\ProductPortal\API\Cartridge.pm'), 'patch affected file.');
    ok ($TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\Ebay\Database\XML\ActionsEbayAuction.xml'), 'patch affected file.');
    ok ($TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\FactFinder\Database\mysql\Tables\factfinderchannel.sql'), 'patch affected file.');
    ok ($TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\IGFS\Patches\Patch_6_12_3\patch.xml'), 'patch affected file.');
    ok (not ($TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\CreditCheck\Templates\MBO\MBO-CreditCheckAgencyConfigCustomerGroups.TabPage.en.xml')), 'no patch affected file.');
    ok (not ($TagHandler->patchAffectedFile('RND/epages/epages5/AllOS/Languages/ca/Cartridges/DE_EPAGES/Presentation/Database/XML/Translation.HelptopicsShop.ca.xml')), 'no patch affected file.');

    ok (not($TagHandler->inspectionAffectedFile('RND/epages/epages5/AllOS/Patch/src/Shared/Config/Scheduler.conf-patch')), 'no inspection affected file.');
    ok ($TagHandler->inspectionAffectedFile('RND/epages/epages5/AllOS/Languages/ca/Cartridges/DE_EPAGES/Presentation/Database/XML/Translation.HelptopicsShop.ca.xml'), 'inspection affected file.');


    #get hot fix tags
    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [
            'release_testtag_88_88_88',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );

    my $hInfo = {
        'engineer' => 'cbordis',
        'jiratask' => 'EPG-19048'
    };

    is_deeply (
        $TagHandler->getInfoForTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88'),
        $hInfo,
        'correct engineer and jira task'
    );

    $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.1', '-F' => '' },
        [
            'patch_test_CVSHandler_end',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );

    $hInfo = {
        'engineer' => 'cbordis',
        'jiratask' => 'EPG-19048'
    };

    is_deeply (
        $TagHandler->getInfoForTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_end'),
        $hInfo,
        'correct engineer and jira task'
    );

    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [
            'release_testtag_99_99_99',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );
    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [
            'release_testtag_99_99_99_1',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );
    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [
            'release_testtag_99_99_99_1_1',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );
    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [
            'release_testtag_99_99_99_2',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );
    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [
            'release_testtag_99_99_99_3',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );
    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [
            'release_testtag_99_99_99_10',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );

    ok (($TagHandler->getLatestCorrespondingHotFixTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88') eq 'release_testtag_88_88_88') , 'return release tag if no hot fix tag exists.');
    ok ( ($TagHandler->getLatestCorrespondingHotFixTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99') eq 'release_testtag_99_99_99_10') , 'correct result for release tag');

    #revision test
    ok(($TagHandler->getRevision('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_10') eq '1.3'), 'correct revision');

    Cleanup($TagHandler);
}

sub Cleanup {
    my ($TagHandler) = @_;

    #remove fictional patch tag
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t', 'patch_test_CVSHandler_new');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t', 'release_test_CVSHandler_old');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_1');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_1_1');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_2');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_3');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_10');
    $TagHandler->removeTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_end');

    #set fictional release tag to start revision
    $TagHandler->command(
        'rtag',
        { '-r' => '1.1', '-F' => '' },
        [
            'release_test_CVSHandler_old',
            'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt'
        ]
    );
}

RunScript( Sub => \&Main );

done_testing();

1;
