/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.lightbox
 *
 * @revision		$Revision: 1.6.4.2 $
 */

jQuery.plugin( "ep.ui.lightbox", ["ep.ui.core", "jQuery.fn.size", "jQuery.ui.widget", "ep.ui.dialog", "ep.ui.slides", "ep.ui.thumbbox"], function( $, undefined ){

	var win = $(window),
		winWidth = win.width(),
		winHeight = win.height(),
		body;

	// hight z-index for dialog / lightbox / overlay
	$.ui.dialog.maxZ = $.ui.dialog.maxZ < 10000 ? 10000 : $.ui.dialog.maxZ;

	$.widget( "ep.ui.lightbox", {
		options: {
			images			: "img",			// selector / array of data / data / image node / array of image nodes / jQuery object
			effect			: "standard",		// change-over effect
			controls		: 'visible',		// show controls or not ('auto' / 'visible' / 'hidden')
			caption			: false,			// show caption of images
			count			: false,			// show count of images like (i of n)
			thumbs			: false,			// show thumbnails inside of lightbox
			closeOnEscape	: true,
			closeOnClickOut	: true
		},

		_create: function(){
			var self = this,
				imgs = self.images = ep.ui.imgData(this.options.images)
					.on("lightbox.uiLightbox", self.Proxy("open"))
				//	.on("resize.uiLightbox", self.Proxy("_resize"))
					.on("resizestop.uiLightbox", self.Proxy("_resizestop"));

			imgs.each(function(i, item){
				if( item.node ){
					$(item.node)
						.on("click.uiLightbox",function(event){
							event.preventDefault();
							imgs.current(i)
								.trigger("lightbox");
						});
				}
			});
		},

		_setOption: function( name, value ){
			var self = this;

			if( name == "thumbs" && self.container ){
				self.container.toggleClass("ep-uiLightbox-showThumbs", value && self.images.length > 1);
			}

			self.Inherited(arguments);
		},

		destroy: function(){
			var self = this;

			self.images
				.off(".uiLightbox")
				.each(function(i, item){
					if( item.node ){
						$(item.node).off("click.uiLightbox");
					}
				});

			if( self.container ){
				self.container.remove();
			}

			self.Inherited(arguments);
		},

		_build: function(){
			var self = this,
				o = self.options;

			if( !self.container ){
				self.container = ep('<div class="ep-uiLightbox"/>')
					.outerWidth(win.width()-100,true)
					.outerHeight(win.height()-100,true)
					.appendTo("body")
					.addClass("ui-hidden");

				// set visibility of thumbs area
				self._setOption( "thumbs", o.thumbs );

				self.slidesWrap = ep('<div class="ep-uiLightbox-slides"/>')
					.appendTo(self.container)
					.uiSlides({
						images	: o.images,
						effect	: o.effect,
						controls: o.controls,
						caption	: o.caption,
						count	: o.count,
						autoplay: false,
						defaultSize: 'L'
					});

				self.thumbsWrap = ep('<div class="ep-uiLightbox-thumbs"/>')
					.appendTo(self.container)
					.uiThumbbox({
						images	: o.images,
						width	: 100,
						height	: 100
					});

				self.ctrlClose = $('<div class="ep-uiLightbox-ctrlClose"/>')
					.prependTo(self.container);

				self.ctrlCloseHandler = $('<a href="javascript:void(0);">X</a>')
					.appendTo(self.ctrlClose)
					.on("click.uiLightbox",self.Proxy("close"));
			}
		},

		_resize: function(){
			if( this.container ){
				body = body && body.length ? body : $('body');

				this.container
					.css({
						top		: (body.prop("scrollTop")+50)+"px",
						left	: (body.prop("scrollLeft")+50)+"px"
					});
			}
		},

		_resizestop: function(){
			if( this.container ){
				this.container
					.outerWidth( win.width() - 100, true )
					.outerHeight( win.height() - 100, true );
			}
		},

		open: function(){
			this._build();

			var self = this;
			self.overlay = new $.ui.dialog.overlay(self);
			self.moveToTop();
			self._resize();
			self.container.removeClass("ui-hidden");
		},

		close: function(){
			var self = this;
			if( self.overlay ){
				self.container.addClass("ui-hidden");
				self.overlay.destroy();
				self.overlay = undefined;
			}

		},

		moveToTop: function() {
			var self = this;

			if (self.overlay) {
				$.ui.dialog.maxZ += 1;
				self.overlay.$el.css("z-index", $.ui.dialog.overlay.maxZ = $.ui.dialog.maxZ);
			}
			self.container.css("z-index", ++$.ui.dialog.maxZ);
		}
	});

});